import $ from 'jquery';

import Form from 'ep-form/js/Form';

/**
 * Obrazac za dodavanje proizvoda u košaricu.
 */
export default class AddToCart extends Form
{
    /**
     * Konstruktor.
     *
     * @param el
     */
    constructor(el)
    {
        super(el);

        this.action = this.$el.data('ajax-action');
        this.productId = parseInt(this.$el.data('product-id'));
        this.type = this.$el.data('product-type');
        this.url = this.$el.data('ajax-url');

        if (this.type === 'grouped') {
            this.quantity = this.$el.find('.quantity-input-list')[0].ep;
        } else {
            this.quantity = this.$el.find('.quantity-input')[0].ep;
        }

        $(this.quantity).on('change', this.onQuantityChange.bind(this));

        if (this.type === 'variable') {
            this.attributes = this.$el.find('.product-attributes')[0].ep;
            this.variationId = 0;
            this.variations = this.$el.data('product-variations');
            this.$variationDisplay = this.$el.find('.add-to-cart__variation');

            $(this.attributes).on('change', this.onVariationChange.bind(this));
        }

        this.updateButton();
    }

    /**
     * Trenutno odabrana varijacija.
     */
    get variation()
    {
        if (!this.attributes.valid) {
            return;
        }

        return this.variations[this.attributes.variationKey];
    }

    /**
     * Kada se promijeni varijacija.
     */
    onVariationChange()
    {
        if (this.attributes.valid) {
            let variation = this.variation;

            if (variation !== undefined) {
                let template = wp.template('variation-template'),
                    html = template({
                        variation: variation
                    });

                this.variationId = parseInt(variation.variation_id);
                this.$variationDisplay.html(html);
            } else {
                let html = wp.template('unavailable-variation-template')();
                this.variationId = 0;
                this.$variationDisplay.html(html);
            }
        } else {
            this.$variationDisplay.html('');
        }

        this.updateButton();
    }

    /**
     * Kada se promijeni odabrana količina.
     */
    onQuantityChange()
    {
        this.updateButton();
    }

    /**
     * Dodaje proizvode u košaricu ajaxom.
     *
     * @param e
     */
    onSubmit(e)
    {
        e.preventDefault();

        let data = {
            action: this.action,
            form_id: this.$el.attr('id'),
            type: this.type,
            products: {}
        }

        switch (this.type) {
            case 'grouped':
                data.products = this.quantity.values;
                break;
            case 'variable':
                data.products[this.variationId] = this.quantity.value;
                break;
            case 'simple':
            default:
                data.products[this.productId] = this.quantity.value;
        }

        this.submit.$el.addClass('loading');

        // woocommerce
        $(document.body).trigger('adding_to_cart', this.submit.$el, data);

        if (this.xhr !== undefined) {
            this.xhr.abort();
        }

        this.xhr = $.post(this.url, data, function(response) {
            this.xhr = undefined;

            this.submit.$el.removeClass('loading');
            this.quantity.reset();
            this.updateButton();

            // woocommerce
            $(document.body).trigger('added_to_cart', [response.fragments, response.cart_hash]);

            // woocommerce
            $(document.body).trigger('wc_cart_button_updated', [this.submit.$el]);
        }.bind(this));
    }

    /**
     * Ažurira submit button.
     */
    updateButton()
    {
        let enabled = this.quantity.valid;

        if (this.type === 'variable') {
            enabled = enabled && this.variation !== undefined;
        }

        if (this.submit !== undefined) {
            this.submit.enabled = enabled;
        }
    }
}