<?php
namespace Ep\Woo\Plugin\Order;

use Ep\Wp\Plugin\PluginInterface;

/**
 * Dodajemo custom statuse narudžbi potrebne Gathu.
 *
 * @package Ep\Woo\Order
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class CustomStatus implements PluginInterface
{
    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function __invoke(): void
    {
        add_action('init', [$this, 'init']);
        add_filter('wc_order_statuses', [$this, 'wc_order_statuses']);
    }

    /**
     * Inicijalizacija.
     */
    public function init(): void
    {
        register_post_status('wc-cekanje-uplate', [
            'label' => 'Čekanje uplate',
            'public' => true,
            'exclude_from_search' => false,
            'show_in_admin_all_list' => true,
            'show_in_admin_status_list' => true,
            'label_count' => 'Čeka se uplata <span class="count">(%s)</span>',
        ]);

        register_post_status('wc-pakiranje', [
            'label' => 'Pakiranje',
            'public' => true,
            'exclude_from_search' => false,
            'show_in_admin_all_list' => true,
            'show_in_admin_status_list' => true,
            'label_count' => 'U pakiranju <span class="count">(%s)</span>',
        ]);
    }

    /**
     * Filtrira statuse narudžbi.
     *
     * @param array $statuses
     *
     * @return array
     */
    public function wc_order_statuses(array $statuses): array
    {
        $new = [];

        foreach ($statuses as $name => $status) {
            $new[$name] = $status;

            // Naše statuse dodajemo nakon 'Na čekanju'
            if ('wc-on-hold' === $name) {
                $new['wc-cekanje-uplate'] = 'Čekanje uplate';
                $new['wc-pakiranje']      = 'Pakiranje';
            }
        }

        return $new;
    }
}