<?php
namespace Ep\Woo\View;

use Ep\Wp\View\AbstractView as BaseAbstractView;

/**
 * Osnovna klasa za WooCommerce viewove.
 *
 * @package Mishe\View
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
abstract class AbstractView extends BaseAbstractView
{
    /** @var bool */
    static $assetsAdded = false;
    /** @var array */
    protected $templateOverrides = [];
    /** @var array */
    protected $templatePartOverrides = [];

    /**
     * Dodaje akcije i filtere u WordPress.
     */
    public function init(): void
    {
        parent::init();

        add_filter('wc_get_template', [$this, 'wc_get_template'], PHP_INT_MAX, 2);
        add_action('woocommerce_before_template_part', [$this, 'wc_before_template_part'], -1, 4);

        if (false === self::$assetsAdded) {
            add_action('wp_enqueue_scripts', [$this, 'wp_enqueue_scripts'], 100);
            self::$assetsAdded = true;
        }
    }

    /**
     * Dodaje front-end resurse.
     */
    public function wp_enqueue_scripts()
    {
        wp_deregister_script('wc-add-to-cart');
        wp_deregister_script('wc-add-to-cart-variation');
        wp_enqueue_script('wp-util');
    }

    /**
     * Okida se prije nego što će WooCommerce učitati template.
     *
     * @param string $located
     * @param string $template_name
     *
     * @return string
     */
    public function wc_get_template(string $located, string $template_name): string
    {
        if (isset($this->templateOverrides[$template_name]) || in_array($template_name, $this->templateOverrides)) {
            return __DIR__ . '/_blank.php';
        }

        return $located;
    }

    /**
     * Renderira zamijenjeni template.
     *
     * @param string $template_name
     */
    public function wc_before_template_part(string $template_name): void
    {
        if (in_array($template_name, $this->templateOverrides)) {
            $callback = 'output';
        } else if (!($callback = $this->templateOverrides[$template_name] ?? false)) {
            return;
        }

        if (!is_callable($callback) && method_exists($this, $callback)) {
            $callback = [$this, $callback];
        }

        call_user_func_array($callback, func_get_args());
    }
}